@@ -1,5 +1,8 @@
-from helpers import validate_params, validate_blogname
-from request import TumblrRequest
+from __future__ import absolute_import
+from builtins import str
+from builtins import object
+from .helpers import validate_params, validate_blogname
+from .request import TumblrRequest
 
 
 class TumblrRestClient(object):
@@ -7,7 +10,7 @@ class TumblrRestClient(object):
     A Python Client for the Tumblr API
     """
 
-    def __init__(self, consumer_key, consumer_secret="", oauth_token="", oauth_secret="", host="http://api.tumblr.com"):
+    def __init__(self, consumer_key, consumer_secret="", oauth_token="", oauth_secret="", host="https://api.tumblr.com"):
         """
         Initializes the TumblrRestClient object, creating the TumblrRequest
         object which deals with all request formatting.
@@ -21,7 +24,7 @@ class TumblrRestClient(object):
         :param oauth_secret: a string, the user specific secret, received
                              from the /access_token endpoint
         :param host: the host that are you trying to send information to,
-                     defaults to http://api.tumblr.com
+                     defaults to https://api.tumblr.com
 
         :returns: None
         """
@@ -39,26 +42,28 @@ class TumblrRestClient(object):
     def avatar(self, blogname, size=64):
         """
         Retrieves the url of the blog's avatar
-        
+
         :param blogname: a string, the blog you want the avatar for
-        
+
         :returns: A dict created from the JSON response
         """
-        url = "/v2/blog/%s/avatar/%d" % (blogname, size)
+        url = "/v2/blog/{}/avatar/{}".format(blogname, size)
         return self.send_api_request("get", url)
 
     def likes(self, **kwargs):
         """
         Gets the current given user's likes
         :param limit: an int, the number of likes you want returned
-        :param offset: an int, the like you want to start at, for pagination.
+        (DEPRECATED) :param offset: an int, the like you want to start at, for pagination.
+        :param before: an int, the timestamp for likes you want before.
+        :param after: an int, the timestamp for likes you want after.
 
             # Start at the 20th like and get 20 more likes.
             client.likes({'offset': 20, 'limit': 20})
 
         :returns: A dict created from the JSON response
         """
-        return self.send_api_request("get", "/v2/user/likes", kwargs, ["limit", "offset"])
+        return self.send_api_request("get", "/v2/user/likes", kwargs, ["limit", "offset", "before", "after"])
 
     def following(self, **kwargs):
         """
@@ -104,7 +109,7 @@ class TumblrRestClient(object):
         """
         kwargs.update({'tag': tag})
         return self.send_api_request("get", '/v2/tagged', kwargs, ['before', 'limit', 'filter', 'tag', 'api_key'], True)
-    
+
     @validate_blogname
     def posts(self, blogname, type=None, **kwargs):
         """
@@ -116,17 +121,18 @@ class TumblrRestClient(object):
         :param tag: a string, the tag you are looking for on posts
         :param limit: an int, the number of results you want
         :param offset: an int, the offset of the posts you want to start at.
+        :param before: an int, the timestamp for posts you want before.
         :param filter: the post format you want returned: HTML, text or raw.
         :param type: the type of posts you want returned, e.g. video. If omitted returns all post types.
 
         :returns: a dict created from the JSON response
         """
         if type is None:
-            url = '/v2/blog/%s/posts' % blogname
+            url = '/v2/blog/{}/posts'.format(blogname)
         else:
-            url = '/v2/blog/%s/posts/%s' % (blogname,type)
-        return self.send_api_request("get", url, kwargs, ['id', 'tag', 'limit', 'offset', 'reblog_info', 'notes_info', 'filter', 'api_key'], True)
-    
+            url = '/v2/blog/{}/posts/{}'.format(blogname, type)
+        return self.send_api_request("get", url, kwargs, ['id', 'tag', 'limit', 'offset', 'before', 'reblog_info', 'notes_info', 'filter', 'api_key', 'npf'], True)
+
     @validate_blogname
     def blog_info(self, blogname):
         """
@@ -137,9 +143,28 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response of information
         """
-        url = "/v2/blog/%s/info" % blogname
+        url = "/v2/blog/{}/info".format(blogname)
         return self.send_api_request("get", url, {}, ['api_key'], True)
-    
+
+    @validate_blogname
+    def blog_following(self, blogname, **kwargs):
+        """
+        Gets the publicly exposed list of blogs that a blog follows
+
+        :param blogname: the name of the blog you want to get information on.
+                         eg: codingjester.tumblr.com
+
+        :param limit: an int, the number of blogs you want returned
+        :param offset: an int, the blog to start at, for pagination.
+
+            # Start at the 20th blog and get 20 more blogs.
+            client.blog_following('pytblr', offset=20, limit=20})
+
+        :returns: a dict created from the JSON response
+        """
+        url = "/v2/blog/{}/following".format(blogname)
+        return self.send_api_request("get", url, kwargs, ['limit', 'offset'])
+
     @validate_blogname
     def followers(self, blogname, **kwargs):
         """
@@ -152,24 +177,26 @@ class TumblrRestClient(object):
 
         :returns: A dict created from the JSON response
         """
-        url = "/v2/blog/%s/followers" % blogname
+        url = "/v2/blog/{}/followers".format(blogname)
         return self.send_api_request("get", url, kwargs, ['limit', 'offset'])
-    
+
     @validate_blogname
     def blog_likes(self, blogname, **kwargs):
         """
         Gets the current given user's likes
         :param limit: an int, the number of likes you want returned
-        :param offset: an int, the like you want to start at, for pagination.
+        (DEPRECATED) :param offset: an int, the like you want to start at, for pagination.
+        :param before: an int, the timestamp for likes you want before.
+        :param after: an int, the timestamp for likes you want after.
 
             # Start at the 20th like and get 20 more likes.
             client.blog_likes({'offset': 20, 'limit': 20})
 
         :returns: A dict created from the JSON response
         """
-        url = "/v2/blog/%s/likes" % blogname
-        return self.send_api_request("get", url, kwargs, ['limit', 'offset'], True)
-    
+        url = "/v2/blog/{}/likes".format(blogname)
+        return self.send_api_request("get", url, kwargs, ['limit', 'offset', 'before', 'after'], True)
+
     @validate_blogname
     def queue(self, blogname, **kwargs):
         """
@@ -181,8 +208,8 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/posts/queue" % blogname
-        return self.send_api_request("get", url, kwargs, ['limit', 'offset', 'filter'])
+        url = "/v2/blog/{}/posts/queue".format(blogname)
+        return self.send_api_request("get", url, kwargs, ['limit', 'offset', 'filter', 'npf'])
 
     @validate_blogname
     def drafts(self, blogname, **kwargs):
@@ -192,21 +219,21 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/posts/draft" % blogname
-        return self.send_api_request("get", url, kwargs, ['filter'])
+        url = "/v2/blog/{}/posts/draft".format(blogname)
+        return self.send_api_request("get", url, kwargs, ['filter', 'npf'])
 
     @validate_blogname
     def submission(self, blogname, **kwargs):
         """
-        Gets posts that are currently in the blog's queue
+        Gets posts that are currently in the blog's submission list
 
         :param offset: an int, the post you want to start at, for pagination.
         :param filter: the post format that you want returned: HTML, text, raw.
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/posts/submission" % blogname
-        return self.send_api_request("get", url, kwargs, ["offset", "filter"])
+        url = "/v2/blog/{}/posts/submission".format(blogname)
+        return self.send_api_request("get", url, kwargs, ['offset', 'filter', 'npf'])
 
     @validate_blogname
     def follow(self, blogname):
@@ -279,7 +306,7 @@ class TumblrRestClient(object):
         """
         kwargs.update({"type": "photo"})
         return self._send_post(blogname, kwargs)
-    
+
     @validate_blogname
     def create_text(self, blogname, **kwargs):
         """
@@ -354,7 +381,7 @@ class TumblrRestClient(object):
         :param format: a string, sets the format type of the post. html or markdown
         :param slug: a string, a short text summary to the end of the post url
         :param title: a string, the title of the conversation
-        :param converstaion: a string, the conversation you are posting
+        :param conversation: a string, the conversation you are posting
 
         :returns: a dict created from the JSON response
         """
@@ -415,7 +442,7 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/post/reblog" % blogname
+        url = "/v2/blog/{}/post/reblog".format(blogname)
 
         valid_options = ['id', 'reblog_key', 'comment'] + self._post_valid_options(kwargs.get('type', None))
         if 'tags' in kwargs and kwargs['tags']:
@@ -433,7 +460,7 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/post/delete" % blogname
+        url = "/v2/blog/{}/post/delete".format(blogname)
         return self.send_api_request('post', url, {'id': id}, ['id'])
 
     @validate_blogname
@@ -452,7 +479,7 @@ class TumblrRestClient(object):
 
         :returns: a dict created from the JSON response
         """
-        url = "/v2/blog/%s/post/edit" % blogname
+        url = "/v2/blog/{}/post/edit".format(blogname)
 
         if 'tags' in kwargs and kwargs['tags']:
             # Take a list of tags and make them acceptable for upload
@@ -461,6 +488,23 @@ class TumblrRestClient(object):
         valid_options = ['id'] + self._post_valid_options(kwargs.get('type', None))
         return self.send_api_request('post', url, kwargs, valid_options)
 
+    @validate_blogname
+    def notes(self, blogname, id, **kwargs):
+        """
+        Gets the notes
+
+        :param blogname: a string, the url of the blog that houses the post
+        :param id: a string, the id of the post.
+        :param mode: a string. Undocumented. Automatically added by tumblr but it's use is not yet known.
+        :param before_timestamp: a string, retreives data before this timestamp
+
+        :returns: a dict created from the JSON response
+        """
+        url = "/v2/blog/{}/notes".format(blogname)
+        valid_options = ["id", "mode", "before_timestamp"]
+        kwargs.update({"id":id})
+        return self.send_api_request('get', url, kwargs, valid_options)
+
     # Parameters valid for /post, /post/edit, and /post/reblog.
     def _post_valid_options(self, post_type=None):
         # These options are always valid
@@ -470,11 +514,11 @@ class TumblrRestClient(object):
         if post_type == 'text':
             valid += ['title', 'body']
         elif post_type == 'photo':
-            valid += ['caption', 'link', 'source', 'data']
+            valid += ['caption', 'link', 'source', 'data', 'photoset_layout']
         elif post_type == 'quote':
             valid += ['quote', 'source']
         elif post_type == 'link':
-            valid += ['title', 'url', 'description']
+            valid += ['title', 'url', 'description', 'thumbnail']
         elif post_type == 'chat':
             valid += ['title', 'conversation']
         elif post_type == 'audio':
@@ -495,10 +539,10 @@ class TumblrRestClient(object):
 
         :returns: a dict parsed from the JSON response
         """
-        url = "/v2/blog/%s/post" % blogname
+        url = "/v2/blog/{}/post".format(blogname)
         valid_options = self._post_valid_options(params.get('type', None))
 
-        if 'tags' in params:
+        if len(params.get("tags", [])) > 0:
             # Take a list of tags and make them acceptable for upload
             params['tags'] = ",".join(params['tags'])
 
@@ -517,22 +561,22 @@ class TumblrRestClient(object):
         :returns: a dict parsed from the JSON response
         """
         if needs_api_key:
-            params.update({'api_key': self.request.consumer.key})
+            params.update({'api_key': self.request.consumer_key})
             valid_parameters.append('api_key')
 
-        files = []
+        files = {}
         if 'data' in params:
             if isinstance(params['data'], list):
                 for idx, data in enumerate(params['data']):
-                    with open(data, 'rb') as f:
-                        files.append(('data['+str(idx)+']', data, f.read()))
+                    files['data['+str(idx)+']'] =  open(params['data'][idx], 'rb')
             else:
-                with open(params['data'], 'rb') as f:
-                    files = [('data', params['data'], f.read())]
+                files = {'data': open(params['data'], 'rb')}
             del params['data']
 
         validate_params(valid_parameters, params)
         if method == "get":
             return self.request.get(url, params)
+        elif method == "delete":
+            return self.request.delete(url, params)
         else:
             return self.request.post(url, params, files)
